# Declare top-level shortcuts
from mysqlorm.ORMModel import ORMModel
from mysqlorm.MySQLConnect import MySQLConnect
from mysqlorm.QueryBuilder import QueryBuilder
from mysqlorm.ResaultBuilder import ResaultBuilder